include Rake::DSL

def dotcover(*args, &block)
	dotcover_wrapper = DotCoverWrapper.new(&block)
	task = Proc.new { dotcover_wrapper.run }
	Rake::Task.define_task(*args, &task)
end		

class DotCoverWrapper
	def initialize(&block)
		@block = block;
	end

	def run()			
		configuration = DotCoverConfiguration.new
		@block.call(configuration)

		nunit_command_line_args = "#{configuration.nunit_assemblies.join(' ')} #{configuration.nunit_parameters.join(' ')}" 
		snapshot_output_file = "#{configuration.report_directory}/#{configuration.snapshot_name}"
		report_file_path_and_name = "#{configuration.report_directory}/#{configuration.report_name}"

	  	run_coverage(configuration.executable, configuration.nunit_executable, nunit_command_line_args, snapshot_output_file, configuration.filters)

	  	if (configuration.html_report)
	  		report_html_file = "#{report_file_path_and_name}.html"
	  		create_html_report(configuration.executable, snapshot_output_file, report_html_file)
	  	end

	  	if (configuration.xml_report)
	  		report_xml_file = "#{report_file_path_and_name}.xml"
	  		create_xml_report(configuration.executable, snapshot_output_file, report_xml_file)
	  	end
	end

	private

	def run_coverage(executable, nunit_executable, nunit_command_line_args, snapshot_output_file, filters)
		output = `#{executable} cover /TargetExecutable=#{nunit_executable} /TargetArguments="#{nunit_command_line_args}" /Output=#{snapshot_output_file} /Filters=#{filters}`
		puts output
		raise 'Unit Test Failures!' if unit_test_errors?
	end

	def unit_test_errors?
		$?.to_i != 0
	end

	def create_html_report(executable, snapshot_output_file, report_file)
		create_report(executable, snapshot_output_file, report_file, "HTML")
	end

	def create_xml_report(executable, snapshot_output_file, report_file)
		create_report(executable, snapshot_output_file, report_file, "XML")
	end

	def create_report(executable, snapshot_output_file, report_file, report_type)
		output = `#{executable} report /Source="#{snapshot_output_file}" /Output="#{report_file}" /ReportType="#{report_type}"`
		puts output
	end

end

class DotCoverConfiguration
	attr_accessor 	:executable,
					:nunit_assemblies,
					:nunit_parameters,
					:nunit_executable,
					:report_directory,
					:snapshot_name,
					:report_name,
					:html_report,
					:xml_report,
					:filters
end